@@ -5,18 +5,22 @@ __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import regex, weakref, operator
-from functools import partial
+import operator
+import weakref
+from collections import OrderedDict, deque
 from datetime import timedelta
-from collections import deque, OrderedDict
+from functools import partial
 
-from calibre.constants import preferred_encoding, DEBUG
+import regex
+
+from calibre.constants import DEBUG, preferred_encoding
 from calibre.db.utils import force_to_bool
 from calibre.utils.config_base import prefs
-from calibre.utils.date import parse_date, UNDEFINED_DATE, now, dt_as_local
-from calibre.utils.icu import primary_no_punc_contains, primary_contains, sort_key
-from calibre.utils.localization import lang_map, canonicalize_lang
-from calibre.utils.search_query_parser import SearchQueryParser, ParseException
+from calibre.utils.date import UNDEFINED_DATE, dt_as_local, now, parse_date
+from calibre.utils.icu import lower as icu_lower
+from calibre.utils.icu import primary_contains, primary_no_punc_contains, sort_key
+from calibre.utils.localization import canonicalize_lang, lang_map
+from calibre.utils.search_query_parser import ParseException, SearchQueryParser
 from polyglot.builtins import iteritems, string_or_bytes
 
 CONTAINS_MATCH = 0
@@ -56,36 +60,36 @@ def _match(query, value, matchkind, use_primary_find_in_search=True, case_sensit
     else:
         internal_match_ok = False
     for t in value:
-        try:  # ignore regexp exceptions, required because search-ahead tries before typing is finished
-            if not case_sensitive:
-                t = icu_lower(t)
-            if (matchkind == EQUALS_MATCH):
-                if internal_match_ok:
-                    if query == t:
-                        return True
-                    return sq in [c.strip() for c in t.split('.') if c.strip()]
-                elif query[0] == '.':
-                    if t.startswith(query[1:]):
-                        ql = len(query) - 1
-                        if (len(t) == ql) or (t[ql:ql+1] == '.'):
-                            return True
-                elif query == t:
+        if not case_sensitive:
+            t = icu_lower(t)
+        if matchkind == EQUALS_MATCH:
+            if internal_match_ok:
+                if query == t:
                     return True
-            elif matchkind == REGEXP_MATCH:
-                flags = regex.UNICODE | regex.VERSION1 | regex.FULLCASE | (0 if case_sensitive else regex.IGNORECASE)
+                return sq in (c.strip() for c in t.split('.') if c.strip())
+            elif query[0] == '.':
+                if t.startswith(query[1:]):
+                    ql = len(query) - 1
+                    if len(t) == ql or t[ql:ql+1] == '.':
+                        return True
+            elif query == t:
+                return True
+        elif matchkind == REGEXP_MATCH:
+            flags = regex.UNICODE | regex.VERSION1 | regex.FULLCASE | (0 if case_sensitive else regex.IGNORECASE)
+            try:
                 if regex.search(query, t, flags) is not None:
                     return True
-            elif matchkind == ACCENT_MATCH:
-                if primary_contains(query, t):
+            except regex.error as e:
+                raise ParseException(_('Invalid regular expression: {!r} with error: {}').format(query, str(e)))
+        elif matchkind == ACCENT_MATCH:
+            if primary_contains(query, t):
+                return True
+        elif matchkind == CONTAINS_MATCH:
+            if not case_sensitive and use_primary_find_in_search:
+                if primary_no_punc_contains(query, t):
                     return True
-            elif matchkind == CONTAINS_MATCH:
-                if not case_sensitive and use_primary_find_in_search:
-                    if primary_no_punc_contains(query, t):
-                        return True
-                elif query in t:
-                    return True
-        except regex.error:
-            pass
+            elif query in t:
+                return True
     return False
 # }}}
 
@@ -237,9 +241,12 @@ class NumericSearch:  # {{{
         dt = datatype
 
         if is_many and query in {'true', 'false'}:
-            valcheck = lambda x: True
             if datatype == 'rating':
-                valcheck = lambda x: x is not None and x > 0
+                def valcheck(x):
+                    return (x is not None and x > 0)
+            else:
+                def valcheck(x):
+                    return True
             found = set()
             for val, book_ids in field_iter():
                 if valcheck(val):
@@ -248,14 +255,18 @@ class NumericSearch:  # {{{
 
         if query == 'false':
             if location == 'cover':
-                relop = lambda x,y: not bool(x)
+                def relop(x, y):
+                    return (not bool(x))
             else:
-                relop = lambda x,y: x is None
+                def relop(x, y):
+                    return (x is None)
         elif query == 'true':
             if location == 'cover':
-                relop = lambda x,y: bool(x)
+                def relop(x, y):
+                    return bool(x)
             else:
-                relop = lambda x,y: x is not None
+                def relop(x, y):
+                    return (x is not None)
         else:
             for k, relop in iteritems(self.operators):
                 if query.startswith(k):
@@ -265,8 +276,11 @@ class NumericSearch:  # {{{
                 relop = self.operators['=']
 
             if dt == 'rating':
-                cast = lambda x: 0 if x is None else int(x)
-                adjust = lambda x: x // 2
+                def cast(x):
+                    return (0 if x is None else int(x))
+
+                def adjust(x):
+                    return (x // 2)
             else:
                 # Datatype is empty if the source is a template. Assume float
                 cast = float if dt in ('float', 'composite', 'half-rating', '') else int
@@ -298,7 +312,8 @@ class NumericSearch:  # {{{
             try:
                 v = cast(val)
             except Exception:
-                v = None
+                raise ParseException(
+                        _('Non-numeric value in column {0}: {1}').format(location, val))
             if v:
                 v = adjust(v)
             if relop(v, q):
@@ -656,7 +671,7 @@ class Parser(SearchQueryParser):  # {{{
             matches = set()
             error_string = '*@*TEMPLATE_ERROR*@*'
             template_cache = {}
-            global_vars = {}
+            global_vars = {'_candidates': candidates}
             for book_id in candidates:
                 mi = self.dbcache.get_proxy_metadata(book_id)
                 val = mi.formatter.safe_format(template, {}, error_string, mi,
@@ -943,7 +958,7 @@ class Search:
     def query_is_cacheable(self, sqp, dbcache, query):
         if query:
             for name, value in sqp.get_queried_fields(query):
-                if name == 'template' and '#@#:d:' in value:
+                if name == 'template':
                     return False
                 elif name in dbcache.field_metadata.all_field_keys():
                     fm = dbcache.field_metadata[name]
