@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -7,26 +6,20 @@ import json
 import re
 from collections import defaultdict, namedtuple
 from contextlib import suppress
-from operator import attrgetter
 from functools import wraps
+from operator import attrgetter
+
 from lxml import etree
 
 from calibre import prints
-from calibre.ebooks.metadata import authors_to_string, check_isbn, string_to_authors
+from calibre.ebooks.metadata import authors_to_string, check_isbn, fmt_sidx, string_to_authors
 from calibre.ebooks.metadata.book.base import Metadata
-from calibre.ebooks.metadata.book.json_codec import (
-    decode_is_multiple, encode_is_multiple, object_to_unicode
-)
-from calibre.ebooks.metadata.utils import (
-    create_manifest_item, ensure_unique, normalize_languages, parse_opf,
-    pretty_print_opf
-)
+from calibre.ebooks.metadata.book.json_codec import decode_is_multiple, encode_is_multiple, object_to_unicode
+from calibre.ebooks.metadata.utils import create_manifest_item, ensure_unique, normalize_languages, parse_opf, pretty_print_opf
 from calibre.ebooks.oeb.base import DC, OPF, OPF2_NSMAP
 from calibre.utils.config import from_json, to_json
-from calibre.utils.date import (
-    fix_only_date, is_date_undefined, isoformat, parse_date as parse_date_, utcnow,
-    w3cdtf
-)
+from calibre.utils.date import fix_only_date, is_date_undefined, isoformat, utcnow, w3cdtf
+from calibre.utils.date import parse_date as parse_date_
 from calibre.utils.iso8601 import parse_iso8601
 from calibre.utils.localization import canonicalize_lang
 from polyglot.builtins import iteritems
@@ -192,7 +185,7 @@ def ensure_prefix(root, prefixes, prefix, value=None):
     prefixes[prefix] = value or reserved_prefixes[prefix]
     prefixes = {k:v for k, v in iteritems(prefixes) if reserved_prefixes.get(k) != v}
     if prefixes:
-        root.set('prefix', ' '.join('%s: %s' % (k, v) for k, v in iteritems(prefixes)))
+        root.set('prefix', ' '.join(f'{k}: {v}' for k, v in iteritems(prefixes)))
     else:
         root.attrib.pop('prefix', None)
 
@@ -301,7 +294,7 @@ def set_identifiers(root, prefixes, refines, new_identifiers, force_identifiers=
     metadata = XPath('./opf:metadata')(root)[0]
     for scheme, val in iteritems(new_identifiers):
         ident = metadata.makeelement(DC('identifier'))
-        ident.text = '%s:%s' % (scheme, val)
+        ident.text = f'{scheme}:{val}'
         if package_identifier is None:
             metadata.append(ident)
         else:
@@ -323,7 +316,7 @@ def identifier_writer(name):
         metadata = XPath('./opf:metadata')(root)[0]
         if ival:
             ident = metadata.makeelement(DC('identifier'))
-            ident.text = '%s:%s' % (name, ival)
+            ident.text = f'{name}:{ival}'
             if package_identifier is None:
                 metadata.append(ident)
             else:
@@ -358,10 +351,26 @@ def find_main_title(root, refines, remove_blanks=False):
     return main_title
 
 
+def find_subtitle(root, refines):
+    for title in XPath('./opf:metadata/dc:title')(root):
+        if not title.text or not title.text.strip():
+            continue
+        props = properties_for_id(title.get('id'), refines)
+        q = props.get('title-type') or ''
+        if 'subtitle' in q or 'sub-title' in q:
+            return title
+
+
 @simple_text
 def read_title(root, prefixes, refines):
     main_title = find_main_title(root, refines)
-    return None if main_title is None else main_title.text.strip()
+    if main_title is None:
+        return None
+    ans = main_title.text.strip()
+    st = find_subtitle(root, refines)
+    if st is not None and st is not main_title:
+        ans += ': ' + st.text.strip()
+    return ans
 
 
 @simple_text
@@ -380,6 +389,9 @@ def read_title_sort(root, prefixes, refines):
 
 def set_title(root, prefixes, refines, title, title_sort=None):
     main_title = find_main_title(root, refines, remove_blanks=True)
+    st = find_subtitle(root, refines)
+    if st is not None:
+        remove_element(st, refines)
     if main_title is None:
         m = XPath('./opf:metadata')(root)[0]
         main_title = m.makeelement(DC('title'))
@@ -483,11 +495,17 @@ def read_authors(root, prefixes, refines):
 
 def set_authors(root, prefixes, refines, authors):
     ensure_prefix(root, prefixes, 'marc')
-    for item in XPath('./opf:metadata/dc:creator')(root):
-        props = properties_for_id_with_scheme(item.get('id'), prefixes, refines)
-        opf_role = item.get(OPF('role'))
-        if (opf_role and opf_role.lower() != 'aut') or (props.get('role') and not is_relators_role(props, 'aut')):
-            continue
+    removals = []
+    for role in ('aut', 'edt'):
+        for item in XPath('./opf:metadata/dc:creator')(root):
+            props = properties_for_id_with_scheme(item.get('id'), prefixes, refines)
+            opf_role = item.get(OPF('role'))
+            if (opf_role and opf_role.lower() != role) or (props.get('role') and not is_relators_role(props, role)):
+                continue
+            removals.append(item)
+        if removals:
+            break
+    for item in removals:
         remove_element(item, refines)
     metadata = XPath('./opf:metadata')(root)[0]
     for author in authors:
@@ -804,14 +822,14 @@ def set_series(root, prefixes, refines, series, series_index):
     for meta in XPath('./opf:metadata/opf:meta[@property="belongs-to-collection"]')(root):
         remove_element(meta, refines)
     if series:
-        create_series(root, refines, series, '%.2g' % series_index)
+        create_series(root, refines, series, fmt_sidx(series_index))
 # }}}
 
 # User metadata {{{
 
 
 def dict_reader(name, load=json.loads, try2=True):
-    pq = '%s:%s' % (CALIBRE_PREFIX, name)
+    pq = f'{CALIBRE_PREFIX}:{name}'
 
     def reader(root, prefixes, refines):
         for meta in XPath('./opf:metadata/opf:meta[@property]')(root):
@@ -839,16 +857,31 @@ def dict_reader(name, load=json.loads, try2=True):
 
 
 read_user_categories = dict_reader('user_categories')
-read_author_link_map = dict_reader('author_link_map')
+_read_link_maps = dict_reader('link_maps')
+_read_author_link_map = dict_reader('author_link_map')
 
 
-def dict_writer(name, serialize=dump_dict, remove2=True):
-    pq = '%s:%s' % (CALIBRE_PREFIX, name)
+def read_link_maps(root, prefixes, refines):
+    ans = _read_link_maps(root, prefixes, refines)
+    if ans is not None:
+        return ans
+    ans = _read_author_link_map(root, prefixes, refines)
+    if ans:
+        ans = {k: v for k, v in ans.items() if v}
+        if ans:
+            return {'authors': ans}
+
+
+def dict_writer(name, serialize=dump_dict, remove2=True, extra_remove=''):
+    pq = f'{CALIBRE_PREFIX}:{name}'
 
     def writer(root, prefixes, refines, val):
         if remove2:
             for meta in XPath('./opf:metadata/opf:meta[@name="calibre:%s"]' % name)(root):
                 remove_element(meta, refines)
+        if extra_remove:
+            for meta in XPath('./opf:metadata/opf:meta[@name="calibre:%s"]' % extra_remove)(root):
+                remove_element(meta, refines)
         for meta in XPath('./opf:metadata/opf:meta[@property]')(root):
             prop = expand_prefix(meta.get('property'), prefixes)
             if prop.lower() == pq:
@@ -863,7 +896,7 @@ def dict_writer(name, serialize=dump_dict, remove2=True):
 
 
 set_user_categories = dict_writer('user_categories')
-set_author_link_map = dict_writer('author_link_map')
+set_link_maps = dict_writer('link_maps', extra_remove='author_link_map')
 
 
 def deserialize_user_metadata(val):
@@ -949,18 +982,26 @@ def read_raster_cover(root, prefixes, refines):
                     return href
 
 
-def ensure_is_only_raster_cover(root, prefixes, refines, raster_cover_item_href):
-    for item in XPath('./opf:metadata/opf:meta[@name="cover"]')(root):
-        remove_element(item, refines)
-    for item in items_with_property(root, 'cover-image', prefixes):
-        prop = normalize_whitespace(item.get('properties').replace('cover-image', ''))
+def set_unique_property(property_name, root, prefixes, href):
+    changed = False
+    for item in items_with_property(root, property_name, prefixes):
+        prop = normalize_whitespace(item.get('properties').replace(property_name, ''))
+        changed = True
         if prop:
             item.set('properties', prop)
         else:
             del item.attrib['properties']
     for item in XPath('./opf:manifest/opf:item')(root):
-        if item.get('href') == raster_cover_item_href:
-            item.set('properties', normalize_whitespace((item.get('properties') or '') + ' cover-image'))
+        if item.get('href') == href:
+            changed = True
+            item.set('properties', normalize_whitespace((item.get('properties') or '') + f' {property_name}'))
+    return changed
+
+
+def ensure_is_only_raster_cover(root, prefixes, refines, raster_cover_item_href):
+    for item in XPath('./opf:metadata/opf:meta[@name="cover"]')(root):
+        remove_element(item, refines)
+    set_unique_property('cover-image', root, prefixes, raster_cover_item_href)
 
 # }}}
 
@@ -1020,10 +1061,14 @@ def read_metadata(root, ver=None, return_extra_data=False):
     s, si = read_series(root, prefixes, refines)
     if s:
         ans.series, ans.series_index = s, si
-    ans.author_link_map = read_author_link_map(root, prefixes, refines) or ans.author_link_map
+    ans.link_maps = read_link_maps(root, prefixes, refines) or ans.link_maps
     ans.user_categories = read_user_categories(root, prefixes, refines) or ans.user_categories
     for name, fm in iteritems(read_user_metadata(root, prefixes, refines) or {}):
-        ans.set_user_metadata(name, fm)
+        try:
+            ans.set_user_metadata(name, fm)
+        except Exception:
+            import traceback
+            traceback.print_exc()
     if return_extra_data:
         ans = ans, ver, read_raster_cover(root, prefixes, refines), first_spine_item(root, prefixes, refines)
     return ans
@@ -1071,8 +1116,8 @@ def apply_metadata(root, mi, cover_prefix='', cover_data=None, apply_null=False,
         set_rating(root, prefixes, refines, mi.rating)
     if ok('series'):
         set_series(root, prefixes, refines, mi.series, mi.series_index or 1)
-    if ok('author_link_map'):
-        set_author_link_map(root, prefixes, refines, getattr(mi, 'author_link_map', None))
+    if ok('link_maps'):
+        set_link_maps(root, prefixes, refines, getattr(mi, 'link_maps', None))
     if ok('user_categories'):
         set_user_categories(root, prefixes, refines, getattr(mi, 'user_categories', None))
     # We ignore apply_null for the next two to match the behavior with opf2.py
