@@ -3,26 +3,54 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 '''
 Miscellaneous widgets used in the GUI
 '''
-import re, os
+import os
+import re
 
-from qt.core import (QIcon, QFont, QLabel, QListWidget, QAction, QEvent,
-        QListWidgetItem, QTextCharFormat, QApplication, QSyntaxHighlighter,
-        QCursor, QColor, QWidget, QPixmap, QSplitterHandle, QToolButton,
-        Qt, pyqtSignal, QSize, QSplitter, QPainter, QPageSize, QPrinter,
-        QLineEdit, QComboBox, QPen, QGraphicsScene, QMenu, QStringListModel, QKeySequence,
-        QCompleter, QTimer, QRect, QGraphicsView, QPalette, QClipboard)
+from qt.core import (
+    QApplication,
+    QClipboard,
+    QColor,
+    QComboBox,
+    QCompleter,
+    QCursor,
+    QEvent,
+    QFont,
+    QGraphicsPixmapItem,
+    QGraphicsScene,
+    QGraphicsView,
+    QIcon,
+    QLabel,
+    QLineEdit,
+    QListWidget,
+    QListWidgetItem,
+    QMenu,
+    QPageSize,
+    QPainter,
+    QPalette,
+    QPen,
+    QPixmap,
+    QPrinter,
+    QRect,
+    QSize,
+    QSplitterHandle,
+    QStringListModel,
+    QSyntaxHighlighter,
+    Qt,
+    QTextCharFormat,
+    QWidget,
+    pyqtSignal,
+)
 
-from calibre.constants import iswindows, ismacos
-from calibre.gui2 import (error_dialog, pixmap_to_data, gprefs,
-        warning_dialog)
-from calibre.gui2.filename_pattern_ui import Ui_Form
-from calibre import fit_image, strftime, force_unicode
+from calibre import fit_image, force_unicode, strftime
+from calibre.constants import ismacos, iswindows
 from calibre.ebooks import BOOK_EXTENSIONS
-from calibre.utils.config import prefs, XMLConfig
+from calibre.gui2 import clip_border_radius, error_dialog, gprefs, pixmap_to_data, warning_dialog
+from calibre.gui2.dnd import DownloadDialog, dnd_get_files, dnd_get_image, dnd_get_local_image_and_pixmap, dnd_has_extension, dnd_has_image, image_extensions
+from calibre.gui2.filename_pattern_ui import Ui_Form
 from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndicator
-from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
-    image_extensions, dnd_has_extension, dnd_get_local_image_and_pixmap, DownloadDialog)
-from calibre.utils.localization import localize_user_manual_link
+from calibre.startup import connect_lambda
+from calibre.utils.config import XMLConfig, prefs
+from calibre.utils.localization import _, localize_user_manual_link
 from polyglot.builtins import native_string_type
 
 history = XMLConfig('history')
@@ -254,7 +282,7 @@ class ImageDropMixin:  # {{{
                 d.start_download()
                 if d.err is None:
                     pmap = QPixmap()
-                    with lopen(d.fpath, 'rb') as f:
+                    with open(d.fpath, 'rb') as f:
                         data = f.read()
                     pmap.loadFromData(data)
                     if not pmap.isNull():
@@ -298,6 +326,8 @@ class ImageDropMixin:  # {{{
             self.set_pixmap(pmap)
             self.cover_changed.emit(
                     pixmap_to_data(pmap, format='PNG'))
+            return True
+        return False
 # }}}
 
 
@@ -320,6 +350,7 @@ class ImageView(QWidget, ImageDropMixin):
 
     BORDER_WIDTH = 1
     cover_changed = pyqtSignal(object)
+    draw_empty_border = False
 
     def __init__(self, parent=None, show_size_pref_name=None, default_show_size=False):
         QWidget.__init__(self, parent)
@@ -363,7 +394,15 @@ class ImageView(QWidget, ImageDropMixin):
     def paintEvent(self, event):
         QWidget.paintEvent(self, event)
         pmap = self._pixmap
+        p = QPainter(self)
+        p.setRenderHints(QPainter.RenderHint.Antialiasing | QPainter.RenderHint.SmoothPixmapTransform)
         if pmap.isNull():
+            if self.draw_empty_border:
+                pen = QPen()
+                pen.setWidth(self.BORDER_WIDTH)
+                p.setPen(pen)
+                p.drawRect(self.rect())
+                p.end()
             return
         w, h = pmap.width(), pmap.height()
         ow, oh = w, h
@@ -376,9 +415,8 @@ class ImageView(QWidget, ImageDropMixin):
         x = int(abs(cw - w)/2)
         y = int(abs(ch - h)/2)
         target = QRect(x, y, w, h)
-        p = QPainter(self)
-        p.setRenderHints(QPainter.RenderHint.Antialiasing | QPainter.RenderHint.SmoothPixmapTransform)
-        p.drawPixmap(target, pmap)
+        with clip_border_radius(p, target):
+            p.drawPixmap(target, pmap)
         if self.draw_border:
             pen = QPen()
             pen.setWidth(self.BORDER_WIDTH)
@@ -386,11 +424,20 @@ class ImageView(QWidget, ImageDropMixin):
             p.drawRect(target)
         if self.show_size:
             draw_size(p, target, ow, oh)
-        p.end()
 # }}}
 
+# CoverView {{{
 
-class CoverView(QGraphicsView, ImageDropMixin):  # {{{
+class RoundedPixmap(QGraphicsPixmapItem):
+
+    def paint(self, painter, option, widget):
+        painter.setRenderHint(QPainter.RenderHint.Antialiasing, True)
+        target = self.boundingRect().toAlignedRect()
+        with clip_border_radius(painter, target):
+            painter.drawPixmap(target, self.pixmap())
+
+
+class CoverView(QGraphicsView, ImageDropMixin):
 
     cover_changed = pyqtSignal(object)
 
@@ -410,7 +457,7 @@ class CoverView(QGraphicsView, ImageDropMixin):  # {{{
 
     def set_pixmap(self, pmap):
         self.scene = QGraphicsScene()
-        self.scene.addPixmap(pmap)
+        self.scene.addItem(RoundedPixmap(pmap))
         self.setScene(self.scene)
 
     def set_background(self, brush=None):
@@ -490,6 +537,8 @@ class LineEditECM:  # {{{
         menu = self.createStandardContextMenu()
         menu.addSeparator()
         self.create_change_case_menu(menu)
+        if callable(getattr(self, 'add_items_to_context_menu', None)):
+            menu = self.add_items_to_context_menu(self, menu)
         menu.exec(event.globalPos())
 
     def modify_case_operation(self, func):
@@ -541,6 +590,45 @@ class EnLineEdit(LineEditECM, QLineEdit):  # {{{
 # }}}
 
 
+# LineEditIndicators {{{
+
+def setup_status_actions(self: QLineEdit):
+    self.status_actions = (
+        self.addAction(QIcon.ic('ok.png'), QLineEdit.ActionPosition.TrailingPosition),
+        self.addAction(QIcon.ic('dialog_error.png'), QLineEdit.ActionPosition.TrailingPosition))
+    self.status_actions[0].setVisible(False)
+    self.status_actions[1].setVisible(False)
+
+
+def stylesheet_for_lineedit(ok, selector='QLineEdit') -> str:
+    if ok is None:
+        return ''
+    col = '#50c878' if ok else '#FF2400'
+    return f'{selector} {{ border: 2px solid {col}; border-radius: 3px }}'
+
+
+def update_status_actions(self: QLineEdit, ok, tooltip: str = ''):
+    self.status_actions[0].setVisible(bool(ok))
+    self.status_actions[1].setVisible(not ok)
+    if ok:
+        self.status_actions[0].setToolTip(tooltip)
+    elif ok is None:
+        self.status_actions[1].setVisible(False)
+    else:
+        self.status_actions[1].setToolTip(tooltip)
+    self.setStyleSheet(stylesheet_for_lineedit(ok))
+
+
+class LineEditIndicators:
+
+    def setup_status_actions(self):
+        setup_status_actions(self)
+
+    def update_status_actions(self, ok, tooltip=''):
+        update_status_actions(self, ok, tooltip)
+# }}}
+
+
 class ItemsCompleter(QCompleter):  # {{{
 
     '''
@@ -678,8 +766,8 @@ class HistoryLineEdit(QComboBox):  # {{{
 
     lost_focus = pyqtSignal()
 
-    def __init__(self, *args):
-        QComboBox.__init__(self, *args)
+    def __init__(self, parent=None):
+        QComboBox.__init__(self, parent)
         self.setEditable(True)
         self.setInsertPolicy(QComboBox.InsertPolicy.NoInsert)
         self.setMaxCount(10)
@@ -689,6 +777,16 @@ class HistoryLineEdit(QComboBox):  # {{{
     def setPlaceholderText(self, txt):
         return self.lineEdit().setPlaceholderText(txt)
 
+    def contextMenuEvent(self, event):
+        menu = self.lineEdit().createStandardContextMenu()
+        menu.addSeparator()
+        menu.addAction(_('Clear history'), self.clear_history_default_impl)
+        menu.exec(event.globalPos())
+
+    def clear_history_default_impl(self):
+        self.clear()
+        history.set(self.store_name, [])
+
     @property
     def store_name(self):
         return 'lineedit_history_'+self._name
@@ -978,13 +1076,12 @@ class PythonHighlighter(QSyntaxHighlighter):  # {{{
 
 # Splitter {{{
 
-
 class SplitterHandle(QSplitterHandle):
 
     double_clicked = pyqtSignal(object)
 
     def __init__(self, orientation, splitter):
-        QSplitterHandle.__init__(self, orientation, splitter)
+        super().__init__(orientation, splitter)
         splitter.splitterMoved.connect(self.splitter_moved,
                 type=Qt.ConnectionType.QueuedConnection)
         self.double_clicked.connect(splitter.double_clicked,
@@ -1002,267 +1099,6 @@ class SplitterHandle(QSplitterHandle):
         self.double_clicked.emit(self)
 
 
-class LayoutButton(QToolButton):
-
-    def __init__(self, icon, text, splitter=None, parent=None, shortcut=None):
-        QToolButton.__init__(self, parent)
-        self.label = text
-        self.setIcon(QIcon.ic(icon))
-        self.setCheckable(True)
-        self.icname = os.path.basename(icon).rpartition('.')[0]
-
-        self.splitter = splitter
-        if splitter is not None:
-            splitter.state_changed.connect(self.update_state)
-        self.setCursor(Qt.CursorShape.PointingHandCursor)
-        self.shortcut = shortcut or ''
-
-    def update_shortcut(self, action_toggle=None):
-        action_toggle = action_toggle or getattr(self, 'action_toggle', None)
-        if action_toggle:
-            sc = ', '.join(sc.toString(QKeySequence.SequenceFormat.NativeText)
-                                for sc in action_toggle.shortcuts())
-            self.shortcut = sc or ''
-            self.update_text()
-
-    def update_text(self):
-        t = _('Hide {}') if self.isChecked() else _('Show {}')
-        t = t.format(self.label)
-        if self.shortcut:
-            t += f' [{self.shortcut}]'
-        self.setText(t), self.setToolTip(t), self.setStatusTip(t)
-
-    def set_state_to_show(self, *args):
-        self.setChecked(False)
-        self.update_text()
-
-    def set_state_to_hide(self, *args):
-        self.setChecked(True)
-        self.update_text()
-
-    def update_state(self, *args):
-        if self.splitter.is_side_index_hidden:
-            self.set_state_to_show()
-        else:
-            self.set_state_to_hide()
-
-    def mouseReleaseEvent(self, ev):
-        if ev.button() == Qt.MouseButton.RightButton:
-            from calibre.gui2.ui import get_gui
-            gui = get_gui()
-            if self.icname == 'search':
-                gui.iactions['Preferences'].do_config(initial_plugin=('Interface', 'Search'), close_after_initial=True)
-                ev.accept()
-                return
-            tab_name = {'book':'book_details', 'grid':'cover_grid', 'cover_flow':'cover_browser',
-                        'tags':'tag_browser', 'quickview':'quickview'}.get(self.icname)
-            if tab_name:
-                if gui is not None:
-                    gui.iactions['Preferences'].do_config(initial_plugin=('Interface', 'Look & Feel', tab_name+'_tab'), close_after_initial=True)
-                    ev.accept()
-                    return
-        return QToolButton.mouseReleaseEvent(self, ev)
-
-
-class Splitter(QSplitter):
-
-    state_changed = pyqtSignal(object)
-    reapply_sizes = pyqtSignal(object)
-
-    def __init__(self, name, label, icon, initial_show=True,
-            initial_side_size=120, connect_button=True,
-            orientation=Qt.Orientation.Horizontal, side_index=0, parent=None,
-            shortcut=None, hide_handle_on_single_panel=True):
-        QSplitter.__init__(self, parent)
-        self.reapply_sizes.connect(self.setSizes, type=Qt.ConnectionType.QueuedConnection)
-        self.hide_handle_on_single_panel = hide_handle_on_single_panel
-        if hide_handle_on_single_panel:
-            self.state_changed.connect(self.update_handle_width)
-        self.original_handle_width = self.handleWidth()
-        self.resize_timer = QTimer(self)
-        self.resize_timer.setSingleShot(True)
-        self.desired_side_size = initial_side_size
-        self.desired_show = initial_show
-        self.resize_timer.setInterval(5)
-        self.resize_timer.timeout.connect(self.do_resize)
-        self.setOrientation(orientation)
-        self.side_index = side_index
-        self._name = name
-        self.label = label
-        self.initial_side_size = initial_side_size
-        self.initial_show = initial_show
-        self.splitterMoved.connect(self.splitter_moved, type=Qt.ConnectionType.QueuedConnection)
-        self.button = LayoutButton(icon, label, self, shortcut=shortcut)
-        if connect_button:
-            self.button.clicked.connect(self.double_clicked)
-
-        if shortcut is not None:
-            self.action_toggle = QAction(QIcon.ic(icon), _('Toggle') + ' ' + label,
-                    self)
-            self.action_toggle.changed.connect(self.update_shortcut)
-            self.action_toggle.triggered.connect(self.toggle_triggered)
-            if parent is not None:
-                parent.addAction(self.action_toggle)
-                if hasattr(parent, 'keyboard'):
-                    parent.keyboard.register_shortcut('splitter %s %s'%(name,
-                        label), str(self.action_toggle.text()),
-                        default_keys=(shortcut,), action=self.action_toggle)
-                else:
-                    self.action_toggle.setShortcut(shortcut)
-            else:
-                self.action_toggle.setShortcut(shortcut)
-
-    def update_shortcut(self):
-        self.button.update_shortcut(self.action_toggle)
-
-    def toggle_triggered(self, *args):
-        self.toggle_side_pane()
-
-    def createHandle(self):
-        return SplitterHandle(self.orientation(), self)
-
-    def initialize(self):
-        for i in range(self.count()):
-            h = self.handle(i)
-            if h is not None:
-                h.splitter_moved()
-        self.state_changed.emit(not self.is_side_index_hidden)
-
-    def splitter_moved(self, *args):
-        self.desired_side_size = self.side_index_size
-        self.state_changed.emit(not self.is_side_index_hidden)
-
-    def update_handle_width(self, not_one_panel):
-        self.setHandleWidth(self.original_handle_width if not_one_panel else 0)
-
-    @property
-    def is_side_index_hidden(self):
-        sizes = list(self.sizes())
-        try:
-            return sizes[self.side_index] == 0
-        except IndexError:
-            return True
-
-    @property
-    def save_name(self):
-        ori = 'horizontal' if self.orientation() == Qt.Orientation.Horizontal \
-                else 'vertical'
-        return self._name + '_' + ori
-
-    def print_sizes(self):
-        if self.count() > 1:
-            print(self.save_name, 'side:', self.side_index_size, 'other:', end=' ')
-            print(list(self.sizes())[self.other_index])
-
-    @property
-    def side_index_size(self):
-        if self.count() < 2:
-            return 0
-        return self.sizes()[self.side_index]
-
-    @side_index_size.setter
-    def side_index_size(self, val):
-        if self.count() < 2:
-            return
-        side_index_hidden = self.is_side_index_hidden
-        if val == 0 and not side_index_hidden:
-            self.save_state()
-        sizes = list(self.sizes())
-        for i in range(len(sizes)):
-            sizes[i] = val if i == self.side_index else 10
-        self.setSizes(sizes)
-        sizes = list(self.sizes())
-        total = sum(sizes)
-        total_needs_adjustment = self.hide_handle_on_single_panel and side_index_hidden
-        if total_needs_adjustment:
-            total -= self.original_handle_width
-        for i in range(len(sizes)):
-            sizes[i] = val if i == self.side_index else total-val
-        self.setSizes(sizes)
-        self.initialize()
-        if total_needs_adjustment:
-            # the handle visibility and therefore size distribution will change
-            # when the event loop ticks
-            self.reapply_sizes.emit(sizes)
-
-    def do_resize(self, *args):
-        orig = self.desired_side_size
-        QSplitter.resizeEvent(self, self._resize_ev)
-        if orig > 20 and self.desired_show:
-            c = 0
-            while abs(self.side_index_size - orig) > 10 and c < 5:
-                self.apply_state(self.get_state(), save_desired=False)
-                c += 1
-
-    def resizeEvent(self, ev):
-        if self.resize_timer.isActive():
-            self.resize_timer.stop()
-        self._resize_ev = ev
-        self.resize_timer.start()
-
-    def get_state(self):
-        if self.count() < 2:
-            return (False, 200)
-        return (self.desired_show, self.desired_side_size)
-
-    def apply_state(self, state, save_desired=True):
-        if state[0]:
-            self.side_index_size = state[1]
-            if save_desired:
-                self.desired_side_size = self.side_index_size
-        else:
-            self.side_index_size = 0
-        self.desired_show = state[0]
-
-    def default_state(self):
-        return (self.initial_show, self.initial_side_size)
-
-    # Public API {{{
-
-    def update_desired_state(self):
-        self.desired_show = not self.is_side_index_hidden
-
-    def save_state(self):
-        if self.count() > 1:
-            gprefs[self.save_name+'_state'] = self.get_state()
-
-    @property
-    def other_index(self):
-        return (self.side_index+1)%2
-
-    def restore_state(self):
-        if self.count() > 1:
-            state = gprefs.get(self.save_name+'_state',
-                    self.default_state())
-            self.apply_state(state, save_desired=False)
-            self.desired_side_size = state[1]
-
-    def toggle_side_pane(self, hide=None):
-        if hide is None:
-            action = 'show' if self.is_side_index_hidden else 'hide'
-        else:
-            action = 'hide' if hide else 'show'
-        getattr(self, action+'_side_pane')()
-
-    def show_side_pane(self):
-        if self.count() < 2 or not self.is_side_index_hidden:
-            return
-        if self.desired_side_size == 0:
-            self.desired_side_size = self.initial_side_size
-        self.apply_state((True, self.desired_side_size))
-
-    def hide_side_pane(self):
-        if self.count() < 2 or self.is_side_index_hidden:
-            return
-        self.apply_state((False, self.desired_side_size))
-
-    def double_clicked(self, *args):
-        self.toggle_side_pane()
-
-    # }}}
-
-# }}}
-
 
 class PaperSizes(QComboBox):  # {{{
 
